'use strict';

var _inherits = require('babel-runtime/helpers/inherits')['default'];

var _classCallCheck = require('babel-runtime/helpers/class-call-check')['default'];

var _objectWithoutProperties = require('babel-runtime/helpers/object-without-properties')['default'];

var _extends = require('babel-runtime/helpers/extends')['default'];

var _interopRequireDefault = require('babel-runtime/helpers/interop-require-default')['default'];

exports.__esModule = true;

var _keycode = require('keycode');

var _keycode2 = _interopRequireDefault(_keycode);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _reactDom = require('react-dom');

var _reactDom2 = _interopRequireDefault(_reactDom);

var _classnames = require('classnames');

var _classnames2 = _interopRequireDefault(_classnames);

var _utilsBootstrapUtils = require('./utils/bootstrapUtils');

var _utilsBootstrapUtils2 = _interopRequireDefault(_utilsBootstrapUtils);

var _reactOverlaysLibRootCloseWrapper = require('react-overlays/lib/RootCloseWrapper');

var _reactOverlaysLibRootCloseWrapper2 = _interopRequireDefault(_reactOverlaysLibRootCloseWrapper);

var _utilsValidComponentChildren = require('./utils/ValidComponentChildren');

var _utilsValidComponentChildren2 = _interopRequireDefault(_utilsValidComponentChildren);

var _utilsCreateChainedFunction = require('./utils/createChainedFunction');

var _utilsCreateChainedFunction2 = _interopRequireDefault(_utilsCreateChainedFunction);

var DropdownMenu = (function (_React$Component) {
  _inherits(DropdownMenu, _React$Component);

  function DropdownMenu() {
    var _this = this;

    _classCallCheck(this, DropdownMenu);

    _React$Component.apply(this, arguments);

    this.handleKeyDown = function (event) {
      switch (event.keyCode) {
        case _keycode2['default'].codes.down:
          _this.focusNext();
          event.preventDefault();
          break;
        case _keycode2['default'].codes.up:
          _this.focusPrevious();
          event.preventDefault();
          break;
        case _keycode2['default'].codes.esc:
        case _keycode2['default'].codes.tab:
          _this.props.onClose(event);
          break;
        default:
      }
    };

    this.focusNext = function () {
      var _getItemsAndActiveIndex = _this.getItemsAndActiveIndex();

      var items = _getItemsAndActiveIndex.items;
      var activeItemIndex = _getItemsAndActiveIndex.activeItemIndex;

      if (items.length === 0) {
        return;
      }

      if (activeItemIndex === items.length - 1) {
        items[0].focus();
        return;
      }

      items[activeItemIndex + 1].focus();
    };

    this.focusPrevious = function () {
      var _getItemsAndActiveIndex2 = _this.getItemsAndActiveIndex();

      var items = _getItemsAndActiveIndex2.items;
      var activeItemIndex = _getItemsAndActiveIndex2.activeItemIndex;

      if (activeItemIndex === 0) {
        items[items.length - 1].focus();
        return;
      }

      items[activeItemIndex - 1].focus();
    };

    this.getItemsAndActiveIndex = function () {
      var items = _this.getFocusableMenuItems();
      var activeElement = document.activeElement;
      var activeItemIndex = items.indexOf(activeElement);

      return { items: items, activeItemIndex: activeItemIndex };
    };

    this.getFocusableMenuItems = function () {
      var menuNode = _reactDom2['default'].findDOMNode(_this);

      if (menuNode === undefined) {
        return [];
      }

      return [].slice.call(menuNode.querySelectorAll('[tabIndex="-1"]'), 0);
    };
  }

  DropdownMenu.prototype.render = function render() {
    var _classes,
        _this2 = this;

    var _props = this.props;
    var children = _props.children;
    var onSelect = _props.onSelect;
    var pullRight = _props.pullRight;
    var className = _props.className;
    var labelledBy = _props.labelledBy;
    var open = _props.open;
    var onClose = _props.onClose;
    var bsRole = _props.bsRole;
    var bsClass = _props.bsClass;

    var props = _objectWithoutProperties(_props, ['children', 'onSelect', 'pullRight', 'className', 'labelledBy', 'open', 'onClose', 'bsRole', 'bsClass']);

    var items = _utilsValidComponentChildren2['default'].map(children, function (child) {
      var childProps = child.props || {};

      return _react2['default'].cloneElement(child, {
        onKeyDown: _utilsCreateChainedFunction2['default'](childProps.onKeyDown, _this2.handleKeyDown),
        onSelect: _utilsCreateChainedFunction2['default'](childProps.onSelect, onSelect)
      }, childProps.children);
    });

    var classes = (_classes = {}, _classes[_utilsBootstrapUtils2['default'].prefix(this.props, 'menu')] = true, _classes[_utilsBootstrapUtils2['default'].prefix(this.props, 'menu-right')] = pullRight, _classes);

    var list = _react2['default'].createElement(
      'ul',
      _extends({
        className: _classnames2['default'](className, classes),
        role: 'menu',
        'aria-labelledby': labelledBy
      }, props),
      items
    );

    if (open) {
      list = _react2['default'].createElement(
        _reactOverlaysLibRootCloseWrapper2['default'],
        { noWrap: true, onRootClose: onClose },
        list
      );
    }

    return list;
  };

  return DropdownMenu;
})(_react2['default'].Component);

DropdownMenu.defaultProps = {
  bsRole: 'menu',
  bsClass: 'dropdown',
  pullRight: false
};

DropdownMenu.propTypes = {
  open: _propTypes2['default'].bool,
  pullRight: _propTypes2['default'].bool,
  onClose: _propTypes2['default'].func,
  labelledBy: _propTypes2['default'].oneOfType([_propTypes2['default'].string, _propTypes2['default'].number]),
  onSelect: _propTypes2['default'].func
};

exports['default'] = DropdownMenu;
module.exports = exports['default'];