'use strict';

var _inherits = require('babel-runtime/helpers/inherits')['default'];

var _createClass = require('babel-runtime/helpers/create-class')['default'];

var _classCallCheck = require('babel-runtime/helpers/class-call-check')['default'];

var _interopRequireDefault = require('babel-runtime/helpers/interop-require-default')['default'];

exports.__esModule = true;

var _request = require('./request');

var _request2 = _interopRequireDefault(_request);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _uid = require('./uid');

var _uid2 = _interopRequireDefault(_uid);

var AjaxUploader = (function (_React$Component) {
  _inherits(AjaxUploader, _React$Component);

  function AjaxUploader() {
    var _this = this;

    _classCallCheck(this, AjaxUploader);

    _React$Component.apply(this, arguments);

    this.state = {
      disabled: false,
      uid: _uid2['default']()
    };

    this.onChange = function (e) {
      if (_this.state.disabled) {
        return;
      }
      _this.setState({
        disabled: true
      });
      var files = e.target.files;
      _this.uploadFiles(files);
    };

    this.onClick = function () {
      var el = _this.file;
      if (!el) {
        return;
      }
      el.click();
    };

    this.onKeyDown = function (e) {
      if (e.key === 'Enter') {
        _this.onClick();
      }
    };

    this.onFileDrop = function (e) {
      if (e.type === 'dragover') {
        e.preventDefault();
        return;
      }

      if (_this.state.disabled) {
        return;
      }

      var files = e.dataTransfer.files;
      _this.uploadFiles(files);

      e.preventDefault();
    };

    this.uploadFiles = function (files) {
      var postFiles = Array.prototype.slice.call(files);
      if (!_this.props.multiple) {
        postFiles = postFiles.slice(0, 1);
      }
      var len = postFiles.length;
      if (len > 0) {
        for (var i = 0; i < len; i++) {
          var file = postFiles[i];
          file.uid = _uid2['default']();
          _this.upload(file);
        }
        if (_this.props.multiple) {
          _this.props.onStart(postFiles);
        } else {
          _this.props.onStart(postFiles[0]);
        }
      }
    };

    this.upload = function (file) {
      var props = _this.props;
      if (!props.beforeUpload) {
        return _this.post(file);
      }

      var before = props.beforeUpload(file);
      if (before && before.then) {
        before.then(function (processedFile) {
          if (Object.prototype.toString.call(processedFile) === '[object File]') {
            _this.post(processedFile);
          } else {
            _this.post(file);
          }
        }, function () {
          _this._reset();
        });
      } else if (before !== false) {
        _this.post(file);
      } else {
        // fix https://github.com/ant-design/ant-design/issues/1989
        _this._reset();
      }
    };

    this.post = function (file) {
      var props = _this.props;
      var data = props.data;
      if (typeof data === 'function') {
        data = data(file);
      }

      _request2['default']({
        action: props.action,
        filename: props.name,
        file: file,
        data: data,
        headers: props.headers,
        withCredentials: props.withCredentials,
        onProgress: function onProgress(e) {
          props.onProgress(e, file);
        },
        onSuccess: function onSuccess(ret) {
          props.onSuccess(ret, file);
          _this._reset();
        },
        onError: function onError(err, ret) {
          props.onError(err, ret, file);
          _this._reset();
        }
      });
    };

    this._reset = function () {
      _this.setState({
        disabled: false,
        uid: _uid2['default']()
      });
    };
  }

  AjaxUploader.prototype.render = function render() {
    var _this2 = this;

    var props = this.props;
    return _react2['default'].createElement(
      'span',
      {
        onClick: this.onClick,
        onKeyDown: this.onKeyDown,
        onDrop: this.onFileDrop,
        onDragOver: this.onFileDrop,
        role: 'button',
        tabIndex: '0',
        className: this.state.disabled ? this.props.prefixCls + ' ' + props.prefixCls + '-disabled' : '' + this.props.prefixCls
      },
      _react2['default'].createElement('input', {
        type: 'file',
        ref: function (node) {
          _this2.file = node;
        },
        key: this.state.uid,
        disabled: this.state.disabled,
        style: { display: 'none' },
        accept: props.accept,
        multiple: this.props.multiple,
        onChange: this.onChange
      }),
      props.children
    );
  };

  _createClass(AjaxUploader, null, [{
    key: 'propTypes',
    value: {
      prefixCls: _propTypes2['default'].string,
      multiple: _propTypes2['default'].bool,
      onStart: _propTypes2['default'].func,
      data: _propTypes2['default'].oneOfType([_propTypes2['default'].object, _propTypes2['default'].func]),
      headers: _propTypes2['default'].object,
      beforeUpload: _propTypes2['default'].func,
      withCredentials: _propTypes2['default'].bool
    },
    enumerable: true
  }]);

  return AjaxUploader;
})(_react2['default'].Component);

exports['default'] = AjaxUploader;
module.exports = exports['default'];