'use strict';

var _inherits = require('babel-runtime/helpers/inherits')['default'];

var _createClass = require('babel-runtime/helpers/create-class')['default'];

var _classCallCheck = require('babel-runtime/helpers/class-call-check')['default'];

var _extends = require('babel-runtime/helpers/extends')['default'];

var _interopRequireDefault = require('babel-runtime/helpers/interop-require-default')['default'];

exports.__esModule = true;

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _reactDom = require('react-dom');

var _reactDom2 = _interopRequireDefault(_reactDom);

var _uid = require('./uid');

var _uid2 = _interopRequireDefault(_uid);

var _warning = require('warning');

var _warning2 = _interopRequireDefault(_warning);

var iframeStyle = {
  position: 'absolute',
  top: '0px',
  opacity: 0,
  filter: 'alpha(opacity=0)',
  left: '0px',
  zIndex: 9999
};

var IframeUploader = (function (_React$Component) {
  _inherits(IframeUploader, _React$Component);

  function IframeUploader() {
    var _this = this;

    _classCallCheck(this, IframeUploader);

    _React$Component.apply(this, arguments);

    this.state = { disabled: false };

    this.onLoad = function () {
      if (_this.state.disabled) {
        return;
      }
      var props = _this.props;
      var response = undefined;
      var eventFile = _this.file;
      try {
        var doc = _this.getIframeDocument();
        var script = doc.getElementsByTagName('script')[0];
        if (script && script.parentNode === doc.body) {
          doc.body.removeChild(script);
        }
        response = doc.body.innerHTML;
        props.onSuccess(response, eventFile);
      } catch (err) {
        process.env.NODE_ENV !== 'production' ? _warning2['default'](false, 'cross domain error for Upload. Maybe server should return document.domain script. see Note from https://github.com/react-component/upload') : undefined;
        response = 'cross-domain';
        props.onError(err, null, eventFile);
      }
      _this.enableIframe();
      _this.initIframe();
    };

    this.onChange = function () {
      var target = _this.getFormInputNode();
      // ie8/9 don't support FileList Object
      // http://stackoverflow.com/questions/12830058/ie8-input-type-file-get-files
      var file = _this.file = {
        uid: _uid2['default'](),
        name: target.value
      };
      _this.props.onStart(_this.getFileForMultiple(file));

      var before = _this.props.beforeUpload(file);
      if (before === false) return;

      var formNode = _this.getFormNode();
      var dataSpan = _this.getFormDataNode();
      var data = _this.props.data;
      if (typeof data === 'function') {
        data = data(file);
      }
      var inputs = [];
      for (var key in data) {
        if (data.hasOwnProperty(key)) {
          inputs.push('<input name="' + key + '" value="' + data[key] + '"/>');
        }
      }
      dataSpan.innerHTML = inputs.join('');
      formNode.submit();
      dataSpan.innerHTML = '';
      _this.disabledIframe();
    };

    this.getIframeNode = function () {
      return _this.iframe;
    };

    this.getIframeDocument = function () {
      return _this.getIframeNode().contentDocument;
    };

    this.getFormNode = function () {
      return _this.getIframeDocument().getElementById('form');
    };

    this.getFormInputNode = function () {
      return _this.getIframeDocument().getElementById('input');
    };

    this.getFormDataNode = function () {
      return _this.getIframeDocument().getElementById('data');
    };

    this.getFileForMultiple = function (file) {
      return _this.props.multiple ? [file] : file;
    };

    this.getIframeHTML = function (domain) {
      var domainScript = '';
      var domainInput = '';
      if (domain) {
        domainScript = '<script>document.domain="' + domain + '";</script>';
        domainInput = '<input name="_documentDomain" value="' + domain + '" />';
      }
      return '\n    <!DOCTYPE html>\n    <html>\n    <head>\n    <meta http-equiv="X-UA-Compatible" content="IE=edge" />\n    <style>\n    body,html {padding:0;margin:0;border:0;overflow:hidden;}\n    </style>\n    ' + domainScript + '\n    </head>\n    <body>\n    <form method="post"\n    encType="multipart/form-data"\n    action="' + _this.props.action + '" id="form" style="display:block;height:9999px;position:relative;overflow:hidden;">\n    <input id="input" type="file"\n     name="' + _this.props.name + '"\n     style="position:absolute;top:0;right:0;height:9999px;font-size:9999px;cursor:pointer;"/>\n    ' + domainInput + '\n    <span id="data"></span>\n    </form>\n    </body>\n    </html>\n    ';
    };

    this.initIframeSrc = function () {
      if (_this.domain) {
        _this.getIframeNode().src = 'javascript:void((function(){\n        var d = document;\n        d.open();\n        d.domain=\'' + _this.domain + '\';\n        d.write(\'\');\n        d.close();\n      })())';
      }
    };

    this.initIframe = function () {
      var iframeNode = _this.getIframeNode();
      var win = iframeNode.contentWindow;
      var doc = undefined;
      _this.domain = _this.domain || '';
      _this.initIframeSrc();
      try {
        doc = win.document;
      } catch (e) {
        _this.domain = document.domain;
        _this.initIframeSrc();
        win = iframeNode.contentWindow;
        doc = win.document;
      }
      doc.open('text/html', 'replace');
      doc.write(_this.getIframeHTML(_this.domain));
      doc.close();
      _this.getFormInputNode().onchange = _this.onChange;
    };

    this.enableIframe = function () {
      if (_this.state.disabled) {
        _this.setState({
          disabled: false
        });
      }
    };

    this.disabledIframe = function () {
      if (!_this.state.disabled) {
        _this.setState({
          disabled: true
        });
      }
    };

    this.updateIframeWH = function () {
      var rootNode = _reactDom2['default'].findDOMNode(_this);
      var iframeNode = _this.getIframeNode();
      iframeNode.style.height = rootNode.offsetHeight + 'px';
      iframeNode.style.width = rootNode.offsetWidth + 'px';
    };
  }

  IframeUploader.prototype.componentDidMount = function componentDidMount() {
    this.updateIframeWH();
    this.initIframe();
  };

  IframeUploader.prototype.componentDidUpdate = function componentDidUpdate() {
    this.updateIframeWH();
  };

  IframeUploader.prototype.render = function render() {
    var _this2 = this;

    var style = _extends({}, iframeStyle, {
      display: this.state.disabled ? 'none' : ''
    });
    return _react2['default'].createElement(
      'span',
      {
        className: this.state.disabled ? this.props.prefixCls + ' ' + this.props.prefixCls + '-disabled' : '' + this.props.prefixCls,
        style: { position: 'relative', zIndex: 0 }
      },
      _react2['default'].createElement('iframe', {
        ref: function (node) {
          _this2.iframe = node;
        },
        onLoad: this.onLoad,
        style: style
      }),
      this.props.children
    );
  };

  _createClass(IframeUploader, null, [{
    key: 'propTypes',
    value: {
      prefixCls: _propTypes2['default'].string,
      onStart: _propTypes2['default'].func,
      multiple: _propTypes2['default'].bool,
      children: _propTypes2['default'].any,
      data: _propTypes2['default'].oneOfType([_propTypes2['default'].object, _propTypes2['default'].func]),
      action: _propTypes2['default'].string,
      name: _propTypes2['default'].string
    },
    enumerable: true
  }]);

  return IframeUploader;
})(_react2['default'].Component);

exports['default'] = IframeUploader;
module.exports = exports['default'];